"""
Metric used in pyMCR
"""

import numpy as _np

def mse(ideal, under_test):
    """
    Mean square error
    """
    return _np.sum((ideal - under_test)**2)/ideal.size

def mean_rel_dif(new, old, only_non_zero=True):
    """ Relative difference """
    
    
    mrd = (new-old)
    
    loc_n0 = _np.where(old != 0.0)
    mrd[loc_n0] = mrd[loc_n0]/old[loc_n0]
    
    if loc_n0[0].size == 0:
        return None
    else:
        if only_non_zero:
            mrd = _np.sum(mrd[loc_n0])
            mrd /= _np.size(new[loc_n0])
        else:
            mrd = _np.sum(mrd)
            mrd /= _np.size(new)
        return mrd
    


"""
Tests using the npmethods
"""

import numpy as np
import numpy.testing
import pytest

class _phantom:
    def __init__(self):
        self.n_colors = 200
        self.wn = np.linspace(400,2800,self.n_colors)

        self.n_components = 3
        sp0 = np.exp(-(self.wn-1200)**2/(2*200**2))
        sp1 = np.exp(-(self.wn-1600)**2/(2*200**2))
        sp2 = np.exp(-(self.wn-2000)**2/(2*200**2))
        self.spectra = np.vstack((sp0, sp1, sp2))

        self.M = 80
        self.N = 120
        self.n_spectra = self.M * self.N

        self.conc = np.zeros((self.M,self.N,self.n_components))

        x0 = 25
        y0 = 25

        x1 = 75
        y1 = 25

        x2 = 50
        y2 = 50

        R = 20

        X,Y = np.meshgrid(np.arange(self.N), np.arange(self.M))

        self.conc[...,0] = np.exp(-(X-x0)**2/(2*R**2))*np.exp(-(Y-y0)**2/(2*R**2))
        self.conc[...,1] = np.exp(-(X-x1)**2/(2*R**2))*np.exp(-(Y-y1)**2/(2*R**2))
        self.conc[...,2] = np.exp(-(X-x2)**2/(2*R**2))*np.exp(-(Y-y2)**2/(2*R**2))

        self.conc /= self.conc.sum(axis=-1)[:,:,None]

        for num in range(self.n_components):
            idx = np.unravel_index(self.conc[...,num].argmax(), self.conc[...,num].shape)
            tmp = np.zeros(3)
            tmp[num] = 1
            self.conc[idx[0],idx[1],:] = 1*tmp
        
        self.hsi = np.dot(self.conc, self.spectra)

@pytest.fixture
def phantom():
    return _phantom()
    
def test_mcr_spectral_guess(phantom):
    """ Basic test with spectral guessing"""

    from pymcr.mcr import McrAls

    mcrals = McrAls(max_iter=200, tol_mse=1e-7, tol_dif_conc=1e-7, 
                    tol_dif_spect=1e-7)

    initial_guess = (phantom.spectra * phantom.wn)
    mcrals.fit(phantom.hsi.reshape((-1,phantom.wn.size)), 
                                   initial_spectra=initial_guess)

    assert mcrals._n_iters < 60
    assert mcrals.mse[-1] < 1e-7
    assert np.abs(mcrals._c_mrd) > 1e-6
    assert np.abs(mcrals._st_mrd) > 1e-6
    assert np.abs(mcrals._c_mrd) < 1e-2
    assert np.abs(mcrals._st_mrd) < 1e-2
    # np.testing.assert_approx_equal(mcrals._c_mrd, 0.000754519972116, significant=3)
    # np.testing.assert_approx_equal(mcrals._st_mrd, -0.00339178932195, significant=3)

def test_mcr_conc_guess(phantom):
    """ Basic test with concentration guessing"""

    from pymcr.mcr import McrAls

    
    initial_guess = np.zeros((phantom.M,phantom.N,phantom.n_components))
    initial_guess[...,0] = np.dot(np.ones(phantom.M)[:,None], (np.arange(phantom.N,0,-1)/phantom.N)[None,:])
    initial_guess[...,1] = np.dot(np.ones(phantom.M)[:,None], (np.arange(phantom.N)/phantom.N)[None,:])
    initial_guess[...,2] = np.dot((np.arange(phantom.M)/phantom.M)[:,None], np.ones(phantom.N)[None,:])
    initial_guess += np.random.rand(phantom.M,phantom.N,3)

    # reshape
    initial_guess = initial_guess.reshape((-1, phantom.n_components))

    mcrals = McrAls(max_iter=200, tol_mse=1e-7, tol_dif_conc=1e-7, 
                    tol_dif_spect=1e-7)
    mcrals.fit(phantom.hsi.reshape((-1,phantom.wn.size)), 
                                   initial_conc=initial_guess)

    assert mcrals._n_iters < 70
    assert mcrals.mse[-1] < 1e-7
    assert np.abs(mcrals._c_mrd) > 1e-6
    assert np.abs(mcrals._st_mrd) > 1e-6
    assert np.abs(mcrals._c_mrd) < 1e-2
    assert np.abs(mcrals._st_mrd) < 1e-2

def test_nnls_spectral_guess(phantom):
    """ Basic test with spectral guessing"""

    from pymcr.mcr import McrAls_NNLS

    mcrals = McrAls_NNLS(max_iter=20, tol_mse=1e-7, tol_dif_conc=1e-7, 
                    tol_dif_spect=1e-7)

    initial_guess = (phantom.spectra * phantom.wn)
    mcrals.fit(phantom.hsi.reshape((-1,phantom.wn.size)), 
                                   initial_spectra=initial_guess)

    assert mcrals._n_iters == 20
    assert mcrals.mse[-1] < 1e-5
    assert np.abs(mcrals._c_mrd) > 1e-6
    assert np.abs(mcrals._st_mrd) > 1e-6
    assert np.abs(mcrals._c_mrd) < 1e-2
    assert np.abs(mcrals._st_mrd) < 1e-2

def test_nnls_conc_guess(phantom):
    """ Basic test with concentration guessing"""

    from pymcr.mcr import McrAls_NNLS

    
    initial_guess = np.zeros((phantom.M,phantom.N,phantom.n_components))
    initial_guess[...,0] = np.dot(np.ones(phantom.M)[:,None], (np.arange(phantom.N,0,-1)/phantom.N)[None,:])
    initial_guess[...,1] = np.dot(np.ones(phantom.M)[:,None], (np.arange(phantom.N)/phantom.N)[None,:])
    initial_guess[...,2] = np.dot((np.arange(phantom.M)/phantom.M)[:,None], np.ones(phantom.N)[None,:])
    initial_guess += np.random.rand(phantom.M,phantom.N,3)

    # reshape
    initial_guess = initial_guess.reshape((-1, phantom.n_components))

    mcrals = McrAls_NNLS(max_iter=20, tol_mse=1e-7, tol_dif_conc=1e-7, 
                    tol_dif_spect=1e-7)
    mcrals.fit(phantom.hsi.reshape((-1,phantom.wn.size)), 
                                   initial_conc=initial_guess)

    assert mcrals._n_iters == 20
    assert mcrals.mse[-1] < 2e-6
    assert np.abs(mcrals._c_mrd) > 1e-6
    assert np.abs(mcrals._st_mrd) > 1e-6
    assert np.abs(mcrals._c_mrd) < 1e-2
    assert np.abs(mcrals._st_mrd) < 1


"""
Testing metrics

"""

import numpy as np
import numpy.testing

from numpy.testing import assert_array_almost_equal

import pymcr.metrics

def test_mse():
    ideal = np.ones(100)
    under_test = np.zeros(100)

    np.testing.assert_equal(pymcr.metrics.mse(ideal, under_test), 1.0)
    np.testing.assert_equal(pymcr.metrics.mse(ideal, ideal), 0.0)

def test_mrd():
    new = np.ones(100)
    old = np.zeros(100)
    old2 = 2*np.ones(100)
    old_combo = np.append(old, old2)

    np.testing.assert_equal(pymcr.metrics.mean_rel_dif(new, old), None)
    np.testing.assert_equal(pymcr.metrics.mean_rel_dif(new, old2), -0.5)
    np.testing.assert_equal(pymcr.metrics.mean_rel_dif(new, new), 0.0)
    np.testing.assert_equal(pymcr.metrics.mean_rel_dif(np.append(new, new),
                                                       old_combo, 
                                                       only_non_zero=True), -0.5)
    np.testing.assert_equal(pymcr.metrics.mean_rel_dif(np.append(new, new),
                                                       old_combo, 
                                                       only_non_zero=False), 0.25)


"""
Simple implementation of Alternating Least-Squares Multivariate Curve Resolution (ALS-MCR)
"""

from timeit import default_timer as _timer
import numpy as _np

from pymcr.abstractmcr import AbstractMcrAls

from pymcr.metrics import mse, mean_rel_dif as mrd
from pymcr.npmethods import ols_c, ols_s, nnls_c, nnls_s


class McrAls(AbstractMcrAls):
    """
    Simple implementation of Alternating Least-Squares Multivariate Curve Resolution (ALS-MCR)
    """
    # alg_list = ['auto', 'pinv', 'cls']

    def __init__(self, tol_dif_spect=1e-6, tol_dif_conc=1e-6, tol_mse=1e-6, max_iter=50,
                 **kwargs):
        """
        MCR-ALS: Multivariate Curve Resolution - Alternating Least Squares

        Parameters
        ----------

        tol_dif_spect : float
            Tolerance of difference between retrieved spectra between iterations.

        tol_dif_conc : float
            Tolerance of difference between retrieved concentrations between iterations.

        tol_mse : float
            Tolerance of mean squared error (MSE) value between iterations.

        max_iter : int
            Maximum number of iterations

        alg : str
            ALS algorithm type. See Notes

        kwargs : dict
            Sets and controls the constraints of the ALS algorithm. See Notes.

        Attributes
        ----------

        n_features : int
            Number of features (for a spectrum, this is the number of frequencies)

        n_samples : int
            Number of provided samples (spectra)

        n_components : int
            Number of endmembers/components to solve for

        mse : ndarray (1D)
            Residual sum-of-squares for each iteration

        conc_ : 2D ndarray ([n_samples, n_components])
            Concentration

        spectra_ : 2D ndarray ([n_components, n_features])


        Methods
        -------

        fit : run the MCR-ALS algorithm

        Notes
        -----
        -   Available algorithm options (alg) are 'auto', 'pinv', and 'cls'
            - 'pinv' solves the least-squares problem using the pseudoinverse
              function in numpy.
            - 'cls' solves the least-squares problem through matrix algebra.
              This is often a faster methods when n_features >> n_components.
            - 'auto' will select the fastest method, which by default is
              'cls' when n_features > 10*n_components.
        -   Available constraints (bool) are 'nonnegative','max_lim', and 'sum_to_one'.
            Additionally, 'max_lim_const' is a modifier if the 'max_lim' = True.
            - 'nonnegative': both conc_ and spectra_ results must be >= 0
            - 'max_lim': conc_ values above 'max_lim_const' (another constraint keyword)
              are set to ='max_lim_const'. Also known as a **closure** constraint.
            - 'sum_to_one': conc_ values across n_components is 1.0 (i.e, the total
              concentration for a given sample is 1.0)

        """
        super().__init__(tol_dif_spect=tol_dif_spect, 
                            tol_dif_conc=tol_dif_conc, 
                            tol_mse=tol_mse, 
                            max_iter=max_iter, **kwargs)

        self.alg_c = ols_c
        self.alg_st = ols_s

class McrAls_NNLS(AbstractMcrAls):
    """
    Simple implementation of Alternating Least-Squares Multivariate Curve Resolution (ALS-MCR)
    using Nonnegative Least Squares approach (CLS)
    """
    # alg_list = ['auto', 'pinv', 'cls']

    def __init__(self, tol_dif_spect=1e-6, tol_dif_conc=1e-6, tol_mse=1e-6, max_iter=50,
                 **kwargs):
        """
        MCR-ALS: Multivariate Curve Resolution - Alternating Least Squares

        Parameters
        ----------

        tol_dif_spect : float
            Tolerance of difference between retrieved spectra between iterations.

        tol_dif_conc : float
            Tolerance of difference between retrieved concentrations between iterations.

        tol_mse : float
            Tolerance of mean squared error (MSE) value between iterations.

        max_iter : int
            Maximum number of iterations

        alg : str
            ALS algorithm type. See Notes

        kwargs : dict
            Sets and controls the constraints of the ALS algorithm. See Notes.

        Attributes
        ----------

        n_features : int
            Number of features (for a spectrum, this is the number of frequencies)

        n_samples : int
            Number of provided samples (spectra)

        n_components : int
            Number of endmembers/components to solve for

        mse : ndarray (1D)
            Residual sum-of-squares for each iteration

        conc_ : 2D ndarray ([n_samples, n_components])
            Concentration

        spectra_ : 2D ndarray ([n_components, n_features])


        Methods
        -------

        fit : run the MCR-ALS algorithm

        Notes
        -----
        -   Available algorithm options (alg) are 'auto', 'pinv', and 'cls'
            - 'pinv' solves the least-squares problem using the pseudoinverse
              function in numpy.
            - 'cls' solves the least-squares problem through matrix algebra.
              This is often a faster methods when n_features >> n_components.
            - 'auto' will select the fastest method, which by default is
              'cls' when n_features > 10*n_components.
        -   Available constraints (bool) are 'nonnegative','max_lim', and 'sum_to_one'.
            Additionally, 'max_lim_const' is a modifier if the 'max_lim' = True.
            - 'nonnegative': both conc_ and spectra_ results must be >= 0
            - 'max_lim': conc_ values above 'max_lim_const' (another constraint keyword)
              are set to ='max_lim_const'. Also known as a **closure** constraint.
            - 'sum_to_one': conc_ values across n_components is 1.0 (i.e, the total
              concentration for a given sample is 1.0)

        """
        super().__init__(tol_dif_spect=tol_dif_spect, 
                            tol_dif_conc=tol_dif_conc, 
                            tol_mse=tol_mse, 
                            max_iter=max_iter, **kwargs)

        self.alg_c = nnls_c
        self.alg_st = nnls_s
        
if __name__ == '__main__':  # pragma: no cover

    n_colors = 200
    wn = _np.linspace(400,2800,n_colors)

    n_components = 3
    sp0 = _np.exp(-(wn-1200)**2/(2*200**2))
    sp1 = _np.exp(-(wn-1600)**2/(2*200**2))
    sp2 = _np.exp(-(wn-2000)**2/(2*200**2))

    M = 80
    N = 120
    n_spectra = M*N

    conc = _np.zeros((M,N,n_components))

    x0 = 25
    y0 = 25

    x1 = 75
    y1 = 25

    x2 = 50
    y2 = 50

    R = 20

    X,Y = _np.meshgrid(_np.arange(N), _np.arange(M))

    conc[...,0] = _np.exp(-(X-x0)**2/(2*R**2))*_np.exp(-(Y-y0)**2/(2*R**2))
    conc[...,1] = _np.exp(-(X-x1)**2/(2*R**2))*_np.exp(-(Y-y1)**2/(2*R**2))
    conc[...,2] = _np.exp(-(X-x2)**2/(2*R**2))*_np.exp(-(Y-y2)**2/(2*R**2))

    conc /= conc.sum(axis=-1)[:,:,None]

    for num in range(n_components):
        idx = _np.unravel_index(conc[...,num].argmax(), conc[...,num].shape)
        tmp = _np.zeros(3)
        tmp[num] = 1
        conc[idx[0],idx[1],:] = 1*tmp
    
    spectra = _np.vstack((sp0, sp1, sp2))
    hsi = _np.dot(conc, spectra)

    mcrals = McrAls_NNLS(max_iter=10, tol_mse=1e-7, tol_dif_conc=1e-6, tol_dif_spect=1e-8)
    # print(mcrals.__dict__)
    mcrals.fit(hsi.reshape((-1,wn.size)), initial_spectra=(spectra*wn))
    # mcrals.fit(hsi.reshape((-1,wn.size)), initial_conc=conc.reshape((-1,n_components)))
    print(mcrals._c_mrd)
    print(mcrals._st_mrd)
    print(mcrals._tmr)




"""
Simple implementation of Alternating Least-Squares Multivariate Curve Resolution (ALS-MCR)
"""

from timeit import default_timer as _timer
import numpy as _np

from pymcr.metrics import mse, mean_rel_dif as mrd

class AbstractMcrAls:
    """Simple implementation of Alternating Least-Squares Multivariate Curve Resolution (ALS-MCR)"""

    def __init__(self, tol_dif_spect=1e-6, tol_dif_conc=1e-6, tol_mse=1e-6, 
                 max_iter=50, **kwargs):
        """
        MCR-ALS: Multivariate Curve Resolution - Alternating Least Squares

        Parameters
        ----------

        tol_dif_spect : float
            Tolerance of difference between retrieved spectra between iterations.

        tol_dif_conc : float
            Tolerance of difference between retrieved concentrations between iterations.

        tol_mse : float
            Tolerance of mean squared error (MSE) value between iterations.

        max_iter : int
            Maximum number of iterations

        alg : str
            ALS algorithm. See Notes

        kwargs : dict
            Sets and controls the constraints of the ALS algorithm. See Notes.

        Attributes
        ----------

        n_features : int
            Number of features (for a spectrum, this is the number of frequencies)

        n_samples : int
            Number of provided samples (spectra)

        n_components : int
            Number of endmembers/components to solve for

        mse : ndarray (1D)
            Residual sum-of-squares for each iteration

        conc_ : 2D ndarray ([n_samples, n_components])
            Concentration

        spectra_ : 2D ndarray ([n_components, n_features])


        Methods
        -------

        fit : run the MCR-ALS algorithm

        Notes
        -----
        -   Available constraints (bool) are 'nonnegative','max_lim', and 'sum_to_one'.
            Additionally, 'max_lim_const' is a modifier if the 'max_lim' = True.
            - 'nonnegative': both conc_ and spectra_ results must be >= 0
            - 'max_lim': conc_ values above 'max_lim_const' (another constraint keyword)
              are set to ='max_lim_const'. Also known as a **closure** constraint.
            - 'sum_to_one': conc_ values across n_components is 1.0 (i.e, the total
              concentration for a given sample is 1.0)

        """

        self.tol_dif_spect = tol_dif_spect
        self.tol_dif_conc = tol_dif_conc
        self.tol = tol_mse
        self.max_iter = max_iter

        self.alg_c = None
        self.alg_st = None

        self._st_last = None
        self._st_now = None

        self._c_last = None
        self._c_now = None

        self._initial_conc = None
        self._initial_spectra = None

        self._n_features = None
        self._n_samples = None
        self._n_components = None

        self.constraints = {'nonnegative': True,
                            'max_lim': True,
                            'max_lim_const': 1.0,
                            'sum_to_one': True}

        if len(kwargs):
            self.constraints.update(kwargs)
        else:
            pass

        # Iteration info
        self.mse = None

        self._n_iters = None
        self._c_mrd = None
        self._st_mrd = None
        self._tmr = None

    @property
    def n_features(self):
        return self._n_features

    @property
    def n_samples(self):
        return self._n_samples

    @property
    def n_components(self):
        return self._n_components

    @property
    def conc_(self):
        return self._c_now

    @property
    def spectra_(self):
        return self._st_now

    @property
    def initial_conc(self):
        return self._initial_conc

    @initial_conc.setter
    def initial_conc(self, value):
        n_rows_conc = value.shape[0]

        if  n_rows_conc == self._n_samples:
            self._initial_spectra = None
            self._initial_conc = 1*value
            self._n_components = self._initial_conc.shape[-1]
        else:
            raise ValueError('initial_conc rows ({}) does not match number of \
samples [data rows] ({})'.format(n_rows_conc, self._n_samples))

    @initial_conc.deleter
    def initial_conc(self):
        self._initial_conc = None

    @property
    def initial_spectra(self):
        return self._initial_spectra

    @initial_spectra.setter
    def initial_spectra(self, value):
        n_cols_spectra = value.shape[-1]

        if  n_cols_spectra == self._n_features:
            self._initial_conc = None
            self._initial_spectra = 1*value
            self._n_components = self._initial_spectra.shape[0]
        else:
            raise ValueError('initial_spectra columns ({}) does not match number of \
features [data cols] ({})'.format(n_cols_spectra, self._n_features))

    @initial_spectra.deleter
    def initial_spectra(self):
        self._initial_spectra = None

    def _data_info(self, data_shape):
        """
        Set the number of features and samples based on the input data's shape.

        Parameters
        ----------

        data_shape : list, tuple
            Shape of the input data.
        """

        if len(data_shape) == 2:
            self._n_features = data_shape[-1]
            self._n_samples = data_shape[0]
        else:
            raise ValueError('Only 2D data inputs supported [n_samples, n_features]')

    def fit(self, data, initial_conc=None, initial_spectra=None):
        """
        Parameters
        ----------

        data : 2D ndarray ([n_samples, n_features])

        initial_conc : 2D ndarray ([n_samples, n_components])
            Initial concentration guess

        initial_spectra : 2D ndarray ([n_components, n_features])
            Initial spectra guess


        Note
        ----

        - **Either** an initial_conc or initial_spectra must be provided. **Not** both.
        """

        if (initial_conc is None) & (initial_spectra is None):
            raise TypeError('fit() requires either an initial concentration or spectra estimate')
        elif (initial_conc is not None) & (initial_spectra is not None):
            raise TypeError('fit() requires either an initial concentration or spectra \
estimate, NOT both')

        self._data_info(data.shape)

        if initial_conc is not None:
            self.initial_conc = initial_conc
        elif initial_spectra is not None:
            self.initial_spectra = initial_spectra

        if self._initial_spectra is not None:
            self._st_now = 1*self._initial_spectra
            self._st_last = 1*self._initial_spectra

            self._c_now = _np.zeros((self._n_samples, self._n_components))
            self._c_last = _np.zeros((self._n_samples, self._n_components))
        else:
            self._c_now = 1*self._initial_conc
            self._c_last = 1*self._initial_conc

            self._st_now = _np.zeros((self._n_components, self._n_features))
            self._st_last = _np.zeros((self._n_components, self._n_features))

        self.mse = []
        self._tmr = _timer()
        for num in range(self.max_iter):
            if (num > 0) | ((self._initial_conc is None) & (num == 0)):
                self._c_last *= 0.0
                self._c_last += 1*self._c_now

                self._c_now *= 0.0
                
                self._c_now += self.alg_c(data, self._st_now)
                
                if self.constraints['nonnegative']:
                    self._c_now[_np.where(self._c_now < 0.0)] = 0.0

                if self.constraints['max_lim']:
                    self._c_now[_np.where(self._c_now > self.constraints['max_lim_const'])] = \
                        self.constraints['max_lim_const']

                if self.constraints['sum_to_one']:
                    self._c_now /= self._c_now.sum(axis=-1)[:,None]
            else:
                pass  # Only end up here if given initial concentration guess
            self._st_last *= 0
            self._st_last += 1*self._st_now

            self._st_now *= 0

            self._st_now += self.alg_st(data, self._c_now)
            
            if self.constraints['nonnegative']:
                self._st_now[_np.where(self._st_now < 0.0)] = 0.0

            self.mse.append(mse(data, _np.dot(self._c_now,self._st_now)))
            print('iteration {} : MSE {:.2e}'.format(num+1, self.mse[-1]))

            self._c_mrd = mrd(self._c_now, self._c_last, only_non_zero=True)
            self._st_mrd = mrd(self._st_now, self._st_last, only_non_zero=True)
            
            if (self.mse[-1] <= self.tol) & (num > 0):
                print('MSE less than tolerance. Finishing...')
                break

            if (self._c_mrd is not None) & (self._st_mrd is not None):
                if (_np.abs(self._c_mrd) <= self.tol_dif_conc) & \
                    (self._c_mrd != 0.0) & (num > 0):
                    if (_np.abs(self._st_mrd) <= self.tol_dif_spect) & \
                    (self._st_mrd != 0.0) & (num > 0):
                        print('Mean rel. diff. in concentration and spectra less \
than tolerances. Finishing...')
                        break
            
        self._tmr -= _timer()
        self._tmr *= -1
        self._n_iters = num+1
        self.mse = _np.array(self.mse)



"""
Numpy-based methods for fitting/regression/inversion
"""

from numpy import dot, zeros
from numpy.linalg import pinv
from scipy.optimize import nnls

def ols_c(H, ST):
    """
    CS^T = H

    Solve for C
    """

    return dot(H, pinv(ST))

def ols_s(H, C):
    """
    CS^T = H

    Solve for S^T
    """
    
    return dot(pinv(C), H)

def nnls_c(H, ST):
    """
    Solution using scipy.optimize.nnls (non-negative least squares)
    """
    temp = zeros((H.shape[0], ST.shape[0]))
    for num, h in enumerate(H):
        # temp[num, :] = nnls(ST.T, h)[0]

        # Maybe faster?
        temp[num, :] = nnls(dot(ST, ST.T), dot(ST, h))[0]
    
    return temp

def nnls_s(H, C):
    """
    Solution using scipy.optimize.nnls (non-negative least squares)
    """
    temp = zeros((C.shape[-1], H.shape[-1]))
    for num, h in enumerate(H.T):
        # temp[:, num] = nnls(C, h)[0]

        # Maybe faster?
        temp[:, num] = nnls(dot(C.T,C), dot(C.T,h))[0]
    
    return temp
